#!/usr/bin/env python
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2016 Michele Segata <segata@ccs-labs.org>
#
# Script for generating DBLP XML file for indexing conference proceedings


# command line argument parsing
import sys
# to read poceedings
import csv


def generate_xml_header(header="header.xml"):
    """Reads and returns the XML header from the header file"""

    with open(header, "r") as header_file:
        content = header_file.readlines()

    return ''.join(content)


def generate_xml_footer(footer="footer.xml"):
    """Reads and returns the XML footer from the footer file"""

    with open(footer, "r") as footer_file:
        content = footer_file.readlines()

    return ''.join(content)


def generate_section(session):
    """Generates a section entry"""

    return '      <section>%s</section>\n' % session


def generate_paper_entry(authors, title, first_page, last_page):
    """Generates the XML entry for a paper. The authors parameter must be a
    string in the following format:
    'Name Surname, Name Surname, Name Surname and Name Surname'
    """
    # open publication
    paper = '      <publ>\n'
    # substitute "and" with a comma for convenience
    authors = authors.replace(' and', ',')
    # generate authors' list
    authors_list = authors.split(', ')
    for author in authors_list:
        paper += '        <author>%s</author>\n' % author

    # add title
    paper += '        <title>%s</title>\n' % title
    # add pages
    paper += '        <pages>%s-%s</pages>\n' % (first_page, last_page)
    # close publication
    paper += '      </publ>\n'
    return paper


def main():
    if len(sys.argv) < 2:
        print('Usage: %s <proceedings file.csv> [output file]' % sys.argv[0])
        return 1

    proceedings = sys.argv[1]
    output_file = 'dblp.xml'
    if len(sys.argv) == 3:
        output_file = sys.argv[2]
    csv_reader = None

    # try reading the csv proceedings
    try:
        proceedings_file = open(proceedings, 'r')
        csv_reader = csv.DictReader(proceedings_file)
    except IOError as ioe:
        print('Unable to open file %s: "%s"' % (proceedings, ioe.message))
        return 1
    except csv.Error as csve:
        print('Error while parsing %s: "%s"' % (proceedings, csve.message))
        return 1
    except Exception as e:
        print('Unkown error while reading %s: "%s"' % (proceedings, e.message))
        return 1

    # start by adding the header
    xml_output = generate_xml_header()
    # init empty session
    current_session = ''

    for paper in csv_reader:
        # add section if needed
        if 'session' in csv_reader.fieldnames:
            if paper['session'] != current_session:
                current_session = paper['session']
                xml_output += generate_section(current_session)
        # add paper
        xml_output += generate_paper_entry(paper['authors'], paper['title'],
                                           paper['page'], paper['lastpage'])

    # add footer
    xml_output += generate_xml_footer()
    # write xml to ouput file
    out = open(output_file, 'w')
    out.write(xml_output)
    out.close()

    return 0


if __name__ == "__main__":
    ret_code = main()
    sys.exit(ret_code)
